@@ -26,7 +26,7 @@ static void recover_one_string(State& state) {
     cerr << "could not parse string" << error << endl;
     return;
   }
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
       std::string_view v;
       error = doc.get(v);
       if (error) {
@@ -58,21 +58,141 @@ static void serialize_twitter(State& state) {
     return;
   }
   size_t bytes = 0;
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     std::string serial = simdjson::minify(doc);
     bytes += serial.size();
     benchmark::DoNotOptimize(serial);
   }
+  // we validate the result
+  {
+    auto serial = simdjson::minify(doc);
+    dom::element doc2; // we parse the minified output
+    if ((error = parser.parse(serial).get(doc2))) { throw std::runtime_error("serialization error"); }
+    auto serial2 = simdjson::minify(doc2); // we minify a second time
+    if(serial != serial2) { throw std::runtime_error("serialization mismatch"); }
+  }
+  // Gigabyte: https://en.wikipedia.org/wiki/Gigabyte
+  state.counters["Gigabytes"] = benchmark::Counter(
+	        double(bytes), benchmark::Counter::kIsRate,
+	        benchmark::Counter::OneK::kIs1000); // For GiB : kIs1024
+  state.counters["docs"] = Counter(double(state.iterations()), benchmark::Counter::kIsRate);
+}
+BENCHMARK(serialize_twitter)->ComputeStatistics("max", [](const std::vector<double>& v) -> double {
+    return *(std::max_element(std::begin(v), std::end(v)));
+  })->DisplayAggregatesOnly(true);
+
+
+static void serialize_big_string_to_string(State& state) {
+  dom::parser parser;
+  std::vector<char> content;
+  content.push_back('\"');
+  for(size_t i = 0 ; i < 100000; i ++) {
+    content.push_back('0' + char(i%10)); // we add what looks like a long list of digits 
+  } 
+  content.push_back('\"');
+  dom::element doc;
+  simdjson::error_code error;
+  if ((error = parser.parse(content.data(), content.size()).get(doc))) {
+    cerr << "could not parse big string" << error << endl;
+    return;
+  }
+  size_t bytes = 0;
+  for (simdjson_unused auto _ : state) {
+    auto serial = simdjson::to_string(doc);
+    bytes += serial.size();
+    benchmark::DoNotOptimize(serial);
+  }
   // Gigabyte: https://en.wikipedia.org/wiki/Gigabyte
   state.counters["Gigabytes"] = benchmark::Counter(
 	        double(bytes), benchmark::Counter::kIsRate,
 	        benchmark::Counter::OneK::kIs1000); // For GiB : kIs1024
   state.counters["docs"] = Counter(double(state.iterations()), benchmark::Counter::kIsRate);
 }
-BENCHMARK(serialize_twitter)->Repetitions(10)->ComputeStatistics("max", [](const std::vector<double>& v) -> double {
+BENCHMARK(serialize_big_string_to_string)->ComputeStatistics("max", [](const std::vector<double>& v) -> double {
     return *(std::max_element(std::begin(v), std::end(v)));
   })->DisplayAggregatesOnly(true);
 
+
+static void serialize_twitter_to_string(State& state) {
+  dom::parser parser;
+  padded_string docdata;
+  auto error = padded_string::load(TWITTER_JSON).get(docdata);
+  if(error) {
+      cerr << "could not parse twitter.json" << error << endl;
+      return;
+  }
+  // we do not want mem. alloc. in the loop.
+  if((error = parser.allocate(docdata.size()))) {
+      cout << error << endl;
+      return;
+  }
+  dom::element doc;
+  if ((error = parser.parse(docdata).get(doc))) {
+    cerr << "could not parse twitter.json" << error << endl;
+    return;
+  }
+  size_t bytes = 0;
+  for (simdjson_unused auto _ : state) {
+    auto serial = simdjson::to_string(doc);
+    bytes += serial.size();
+    benchmark::DoNotOptimize(serial);
+  }
+  // we validate the result
+  {
+    auto serial = simdjson::to_string(doc); 
+    dom::element doc2; // we parse the stringify output
+    if ((error = parser.parse(serial).get(doc2))) { throw std::runtime_error("serialization error"); }
+    auto serial2 = simdjson::to_string(doc2); // we stringify again
+    if(serial != serial2) { throw std::runtime_error("serialization mismatch"); }
+  }
+  // Gigabyte: https://en.wikipedia.org/wiki/Gigabyte
+  state.counters["Gigabytes"] = benchmark::Counter(
+	        double(bytes), benchmark::Counter::kIsRate,
+	        benchmark::Counter::OneK::kIs1000); // For GiB : kIs1024
+  state.counters["docs"] = Counter(double(state.iterations()), benchmark::Counter::kIsRate);
+}
+BENCHMARK(serialize_twitter_to_string)->ComputeStatistics("max", [](const std::vector<double>& v) -> double {
+    return *(std::max_element(std::begin(v), std::end(v)));
+  })->DisplayAggregatesOnly(true);
+
+static void serialize_twitter_string_builder(State& state) {
+  dom::parser parser;
+  padded_string docdata;
+  auto error = padded_string::load(TWITTER_JSON).get(docdata);
+  if(error) {
+      cerr << "could not parse twitter.json" << error << endl;
+      return;
+  }
+  // we do not want mem. alloc. in the loop.
+  if((error = parser.allocate(docdata.size()))) {
+      cout << error << endl;
+      return;
+  }
+  dom::element doc;
+  if ((error = parser.parse(docdata).get(doc))) {
+    cerr << "could not parse twitter.json" << error << endl;
+    return;
+  }
+  size_t bytes = 0;
+  simdjson::internal::string_builder<> sb;// not part of our public API, for internal use
+  for (simdjson_unused auto _ : state) {
+    sb.clear();
+    sb.append(doc);
+    std::string_view serial = sb.str();
+    bytes += serial.size();
+    benchmark::DoNotOptimize(serial);
+  }
+  // Gigabyte: https://en.wikipedia.org/wiki/Gigabyte
+  state.counters["Gigabytes"] = benchmark::Counter(
+	        double(bytes), benchmark::Counter::kIsRate,
+	        benchmark::Counter::OneK::kIs1000); // For GiB : kIs1024
+  state.counters["docs"] = Counter(double(state.iterations()), benchmark::Counter::kIsRate);
+}
+BENCHMARK(serialize_twitter_string_builder)->ComputeStatistics("max", [](const std::vector<double>& v) -> double {
+    return *(std::max_element(std::begin(v), std::end(v)));
+  })->DisplayAggregatesOnly(true);
+
+
 static void numbers_scan(State& state) {
   // Prints the number of results in twitter.json
   dom::parser parser;
@@ -82,7 +202,7 @@ static void numbers_scan(State& state) {
     cerr << "could not read " << NUMBERS_JSON << " as an array: " << error << endl;
     return;
   }
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     std::vector<double> container;
     for (auto e : arr) {
       double x;
@@ -104,7 +224,7 @@ static void numbers_size_scan(State& state) {
     cerr << "could not read " << NUMBERS_JSON << " as an array: " << error << endl;
     return;
   }
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     std::vector<double> container;
     container.resize(arr.size());
     size_t pos = 0;
@@ -130,7 +250,7 @@ static void numbers_type_scan(State& state) {
     cerr << "could not read " << NUMBERS_JSON << " as an array" << endl;
     return;
   }
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     std::vector<double> container;
     for (auto e : arr) {
       dom::element_type actual_type = e.type();
@@ -156,7 +276,7 @@ static void numbers_type_size_scan(State& state) {
     cerr << "could not read " << NUMBERS_JSON << " as an array: " << error << endl;
     return;
   }
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     std::vector<double> container;
     container.resize(arr.size());
     size_t pos = 0;
@@ -181,7 +301,7 @@ static void numbers_load_scan(State& state) {
   dom::parser parser;
   dom::array arr;
   simdjson::error_code error;
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     // this may hit the disk, but probably just once
     if ((error = parser.load(NUMBERS_JSON).get(arr))) {
       cerr << "could not read " << NUMBERS_JSON << " as an array: " << error << endl;
@@ -204,7 +324,7 @@ static void numbers_load_size_scan(State& state) {
   dom::parser parser;
   dom::array arr;
   simdjson::error_code error;
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     // this may hit the disk, but probably just once
     if ((error = parser.load(NUMBERS_JSON).get(arr))) {
       cerr << "could not read " << NUMBERS_JSON << " as an array" << endl;
@@ -233,7 +353,7 @@ static void numbers_exceptions_scan(State& state) {
   // Prints the number of results in twitter.json
   dom::parser parser;
   dom::array arr = parser.load(NUMBERS_JSON);
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     std::vector<double> container;
     for (double x : arr) {
       container.push_back(x);
@@ -248,7 +368,7 @@ static void numbers_exceptions_size_scan(State& state) {
   // Prints the number of results in twitter.json
   dom::parser parser;
   dom::array arr = parser.load(NUMBERS_JSON);
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     std::vector<double> container;
     container.resize(arr.size());
     size_t pos = 0;
@@ -268,7 +388,7 @@ static void numbers_type_exceptions_scan(State& state) {
   // Prints the number of results in twitter.json
   dom::parser parser;
   dom::array arr = parser.load(NUMBERS_JSON);
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     std::vector<double> container;
     for (auto e : arr) {
       dom::element_type actual_type = e.type();
@@ -287,7 +407,7 @@ static void numbers_type_exceptions_size_scan(State& state) {
   // Prints the number of results in twitter.json
   dom::parser parser;
   dom::array arr = parser.load(NUMBERS_JSON);
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     std::vector<double> container;
     container.resize(arr.size());
     size_t pos = 0;
@@ -308,7 +428,7 @@ BENCHMARK(numbers_type_exceptions_size_scan);
 static void numbers_exceptions_load_scan(State& state) {
   // Prints the number of results in twitter.json
   dom::parser parser;
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     // this may hit the disk, but probably just once
     dom::array arr = parser.load(NUMBERS_JSON);
     std::vector<double> container;
@@ -324,7 +444,7 @@ BENCHMARK(numbers_exceptions_load_scan);
 static void numbers_exceptions_load_size_scan(State& state) {
   // Prints the number of results in twitter.json
   dom::parser parser;
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     // this may hit the disk, but probably just once
     dom::array arr = parser.load(NUMBERS_JSON);
     std::vector<double> container;
@@ -345,7 +465,7 @@ static void twitter_count(State& state) {
   // Prints the number of results in twitter.json
   dom::parser parser;
   dom::element doc = parser.load(TWITTER_JSON);
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     uint64_t result_count = doc["search_metadata"]["count"];
     if (result_count != 100) { return; }
   }
@@ -358,7 +478,7 @@ static void iterator_twitter_count(State& state) {
   // Prints the number of results in twitter.json
   padded_string json = padded_string::load(TWITTER_JSON);
   ParsedJson pj = build_parsed_json(json);
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     ParsedJson::Iterator iter(pj);
     // uint64_t result_count = doc["search_metadata"]["count"];
     if (!iter.move_to_key("search_metadata")) { return; }
@@ -376,7 +496,7 @@ static void twitter_default_profile(State& state) {
   // Count unique users with a default profile.
   dom::parser parser;
   dom::element doc = parser.load(TWITTER_JSON);
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     set<string_view> default_users;
     for (dom::object tweet : doc["statuses"]) {
       dom::object user = tweet["user"];
@@ -394,7 +514,7 @@ static void twitter_image_sizes(State& state) {
   dom::parser parser;
   dom::element doc = parser.load(TWITTER_JSON);
   simdjson::error_code error;
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     set<tuple<uint64_t, uint64_t>> image_sizes;
     for (dom::object tweet : doc["statuses"]) {
       dom::array media;
@@ -419,7 +539,7 @@ static void error_code_twitter_count(State& state) noexcept {
   simdjson::error_code error;
   dom::element doc;
   if ((error = parser.load(TWITTER_JSON).get(doc))) { return; }
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     uint64_t value;
     if ((error = doc["search_metadata"]["count"].get(value))) { return; }
     if (value != 100) { return; }
@@ -433,7 +553,7 @@ static void error_code_twitter_default_profile(State& state) noexcept {
   simdjson::error_code error;
   dom::element doc;
   if ((error = parser.load(TWITTER_JSON).get(doc))) { std::cerr << error << std::endl; return; }
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     set<string_view> default_users;
 
     dom::array tweets;
@@ -463,7 +583,7 @@ static void iterator_twitter_default_profile(State& state) {
   auto error = padded_string::load(TWITTER_JSON).get(json);
   if (error) { std::cerr << error << std::endl; return; }
   ParsedJson pj = build_parsed_json(json);
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     set<string_view> default_users;
     ParsedJson::Iterator iter(pj);
 
@@ -504,7 +624,7 @@ static void error_code_twitter_image_sizes(State& state) noexcept {
   simdjson::error_code error;
   dom::element doc;
   if ((error = parser.load(TWITTER_JSON).get(doc))) { std::cerr << error << std::endl; return; }
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     set<tuple<uint64_t, uint64_t>> image_sizes;
     dom::array statuses;
     if ((error = doc["statuses"].get(statuses))) { return; }
@@ -536,7 +656,7 @@ static void iterator_twitter_image_sizes(State& state) {
   auto error = padded_string::load(TWITTER_JSON).get(json);
   if (error) { std::cerr << error << std::endl; return; }
   ParsedJson pj = build_parsed_json(json);
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     set<tuple<uint64_t, uint64_t>> image_sizes;
     ParsedJson::Iterator iter(pj);
 
@@ -601,7 +721,7 @@ static void print_json(State& state) noexcept {
 
   int code = json_parse(json, parser);
   if (code) { cerr << error_message(code) << endl; return; }
-  for (SIMDJSON_UNUSED auto _ : state) {
+  for (simdjson_unused auto _ : state) {
     std::stringstream s;
     if (!parser.print_json(s)) { cerr << "print_json failed" << endl; return; }
   }
