@@ -1,14 +1,13 @@
-#include "fallback/begin_implementation.h"
-#include "fallback/dom_parser_implementation.h"
-#include "generic/stage2/jsoncharutils.h"
+#include "simdjson/fallback/begin.h"
 
 //
 // Stage 1
 //
 #include "generic/stage1/find_next_document_index.h"
 
-namespace {
+namespace simdjson {
 namespace SIMDJSON_IMPLEMENTATION {
+namespace {
 namespace stage1 {
 
 class structural_scanner {
@@ -179,8 +178,9 @@ private:
 }; // structural_scanner
 
 } // namespace stage1
+} // unnamed namespace
 
-SIMDJSON_WARN_UNUSED error_code dom_parser_implementation::stage1(const uint8_t *_buf, size_t _len, bool partial) noexcept {
+simdjson_warn_unused error_code dom_parser_implementation::stage1(const uint8_t *_buf, size_t _len, bool partial) noexcept {
   this->buf = _buf;
   this->len = _len;
   stage1::structural_scanner scanner(*this, partial);
@@ -222,7 +222,7 @@ static uint8_t jump_table[256 * 3] = {
     0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1,
 };
 
-SIMDJSON_WARN_UNUSED error_code implementation::minify(const uint8_t *buf, size_t len, uint8_t *dst, size_t &dst_len) const noexcept {
+simdjson_warn_unused error_code implementation::minify(const uint8_t *buf, size_t len, uint8_t *dst, size_t &dst_len) const noexcept {
   size_t i = 0, pos = 0;
   uint8_t quote = 0;
   uint8_t nonescape = 1;
@@ -240,11 +240,11 @@ SIMDJSON_WARN_UNUSED error_code implementation::minify(const uint8_t *buf, size_
   }
   dst_len = pos; // we intentionally do not work with a reference
   // for fear of aliasing
-  return SUCCESS;
+  return quote ? UNCLOSED_STRING : SUCCESS;
 }
 
 // credit: based on code from Google Fuchsia (Apache Licensed)
-SIMDJSON_WARN_UNUSED bool implementation::validate_utf8(const char *buf, size_t len) const noexcept { 
+simdjson_warn_unused bool implementation::validate_utf8(const char *buf, size_t len) const noexcept { 
   const uint8_t *data = (const uint8_t *)buf;
   uint64_t pos = 0;
   uint64_t next_pos = 0;
@@ -297,7 +297,7 @@ SIMDJSON_WARN_UNUSED bool implementation::validate_utf8(const char *buf, size_t
       code_point =
           (byte & 0b00000111) << 18 | (data[pos + 1] & 0b00111111) << 12 |
           (data[pos + 2] & 0b00111111) << 6 | (data[pos + 3] & 0b00111111);
-      if (code_point < 0xffff || 0x10ffff < code_point) { return false; }
+      if (code_point <= 0xffff || 0x10ffff < code_point) { return false; }
     } else {
       // we may have a continuation
       return false;
@@ -308,33 +308,31 @@ SIMDJSON_WARN_UNUSED bool implementation::validate_utf8(const char *buf, size_t
 }
 
 } // namespace SIMDJSON_IMPLEMENTATION
-} // unnamed namespace
+} // namespace simdjson
 
 //
 // Stage 2
 //
-#include "fallback/stringparsing.h"
-#include "fallback/numberparsing.h"
 #include "generic/stage2/tape_builder.h"
 
-namespace {
+namespace simdjson {
 namespace SIMDJSON_IMPLEMENTATION {
 
-SIMDJSON_WARN_UNUSED error_code dom_parser_implementation::stage2(dom::document &_doc) noexcept {
+simdjson_warn_unused error_code dom_parser_implementation::stage2(dom::document &_doc) noexcept {
   return stage2::tape_builder::parse_document<false>(*this, _doc);
 }
 
-SIMDJSON_WARN_UNUSED error_code dom_parser_implementation::stage2_next(dom::document &_doc) noexcept {
+simdjson_warn_unused error_code dom_parser_implementation::stage2_next(dom::document &_doc) noexcept {
   return stage2::tape_builder::parse_document<true>(*this, _doc);
 }
 
-SIMDJSON_WARN_UNUSED error_code dom_parser_implementation::parse(const uint8_t *_buf, size_t _len, dom::document &_doc) noexcept {
+simdjson_warn_unused error_code dom_parser_implementation::parse(const uint8_t *_buf, size_t _len, dom::document &_doc) noexcept {
   auto error = stage1(_buf, _len, false);
   if (error) { return error; }
   return stage2(_doc);
 }
 
 } // namespace SIMDJSON_IMPLEMENTATION
-} // unnamed namespace
+} // namespace simdjson
 
-#include "fallback/end_implementation.h"
\ No newline at end of file
+#include "simdjson/fallback/end.h"
\ No newline at end of file
