@@ -1,6 +1,8 @@
 #include <unistd.h>
 
-#include "simdjson/jsonparser.h"
+#include "simdjson.h"
+
+SIMDJSON_PUSH_DISABLE_ALL_WARNINGS
 
 // #define RAPIDJSON_SSE2 // bad
 // #define RAPIDJSON_SSE42 // bad
@@ -24,8 +26,10 @@ extern "C" {
 #include "jsoncpp.cpp"
 #include "json/json.h"
 
+SIMDJSON_POP_DISABLE_WARNINGS
+
 // fastjson has a tricky interface
-void on_json_error(void *, const fastjson::ErrorContext &ec) {
+void on_json_error(void *, simdjson_unused const fastjson::ErrorContext &ec) {
   // std::cerr<<"ERROR: "<<ec.mesg<<std::endl;
 }
 bool fastjson_parse(const char *input) {
@@ -60,31 +64,23 @@ int main(int argc, char *argv[]) {
   }
   const char *filename = argv[optind];
   simdjson::padded_string p;
-  try {
-    simdjson::get_corpus(filename).swap(p);
-  } catch (const std::exception &e) { // caught by reference to base
-    std::cout << "Could not load the file " << filename << std::endl;
+  auto error = simdjson::padded_string::load(filename).get(p);
+  if (error) {
+    std::cerr << "Could not load the file " << filename << ": " << error << std::endl;
     return EXIT_FAILURE;
   }
   if (verbose) {
     std::cout << "Input has ";
-    if (p.size() > 1024 * 1024)
-      std::cout << p.size() / (1024 * 1024) << " MB ";
-    else if (p.size() > 1024)
-      std::cout << p.size() / 1024 << " KB ";
+    if (p.size() > 1000 * 1000)
+      std::cout << p.size() / (1000 * 1000) << " MB ";
+    else if (p.size() > 1000)
+      std::cout << p.size() / 1000 << " KB ";
     else
       std::cout << p.size() << " B ";
     std::cout << std::endl;
   }
-  simdjson::ParsedJson pj;
-  size_t max_depth = 1024 * 4;
-  bool allocok = pj.allocate_capacity(p.size(), max_depth);
-  if (!allocok) {
-    std::cerr << "can't allocate memory" << std::endl;
-    return EXIT_FAILURE;
-  }
-  int oursreturn = json_parse(p, pj);
-  bool ours_correct = (oursreturn == 0); // returns 0 on success
+  simdjson::dom::parser parser;
+  error = parser.parse(p).error();
 
   rapidjson::Document d;
 
@@ -100,19 +96,19 @@ int main(int argc, char *argv[]) {
           .is_valid();
   if (just_favorites) {
     printf("our parser                 : %s \n",
-           ours_correct ? "correct" : "invalid");
+           (error == simdjson::error_code::SUCCESS) ? "correct" : "invalid");
     printf("rapid (check encoding)     : %s \n",
            rapid_correct_checkencoding ? "correct" : "invalid");
     printf("sajson                     : %s \n",
            sajson_correct ? "correct" : "invalid");
-    if (oursreturn == simdjson::DEPTH_ERROR) {
+    if (error == simdjson::DEPTH_ERROR) {
       printf("simdjson encountered a DEPTH_ERROR, it was parametrized to "
              "reject documents with depth exceeding %zu.\n",
-             max_depth);
+             parser.max_depth());
     }
-    if ((ours_correct != rapid_correct_checkencoding) ||
+    if (((error == simdjson::error_code::SUCCESS) != rapid_correct_checkencoding) ||
         (rapid_correct_checkencoding != sajson_correct) ||
-        (ours_correct != sajson_correct)) {
+        ((error == simdjson::SUCCESS) != sajson_correct)) {
       printf("WARNING: THEY DISAGREE\n\n");
       return EXIT_FAILURE;
     }
@@ -139,11 +135,11 @@ int main(int argc, char *argv[]) {
   if (tokens == nullptr) {
     printf("Failed to alloc memory for jsmn\n");
   } else {
-    jsmn_parser parser;
-    jsmn_init(&parser);
+    jsmn_parser jsmnparser;
+    jsmn_init(&jsmnparser);
     memcpy(buffer, p.data(), p.size());
     buffer[p.size()] = '\0';
-    int r = jsmn_parse(&parser, buffer, p.size(), tokens.get(), p.size());
+    int r = jsmn_parse(&jsmnparser, buffer, p.size(), tokens.get(), static_cast<unsigned int>(p.size()));
     tokens = nullptr;
     jsmn_correct = (r > 0);
   }
@@ -165,7 +161,7 @@ int main(int argc, char *argv[]) {
   delete json_cpp_reader;
 
   printf("our parser                 : %s \n",
-         ours_correct ? "correct" : "invalid");
+         (err == simdjson::error_code::SUCCESS) ? "correct" : "invalid");
   printf("rapid                      : %s \n",
          rapid_correct ? "correct" : "invalid");
   printf("rapid (check encoding)     : %s \n",
