@@ -1,17 +1,50 @@
-#include <assert.h>
 #include <cstring>
-#include <dirent.h>
-#include <inttypes.h>
-#include <math.h>
-#include <stdbool.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include <cinttypes>
+#include <cmath>
+#include <cstdio>
+#include <cstdlib>
+
+
 
 #ifndef JSON_TEST_NUMBERS
 #define JSON_TEST_NUMBERS
 #endif
 
-#include "simdjson/common_defs.h"
+#if (!(_MSC_VER) && !(__MINGW32__) && !(__MINGW64__))
+#include <dirent.h>
+#else
+#include <dirent_portable.h>
+#endif
+
+void found_invalid_number(const uint8_t *buf);
+void found_float(double result, const uint8_t *buf);
+void found_integer(int64_t result, const uint8_t *buf);
+void found_unsigned_integer(uint64_t result, const uint8_t *buf);
+
+#include "simdjson.h"
+
+
+
+/**
+ * Some systems have bad floating-point parsing. We want to exclude them.
+ */
+#if defined(SIMDJSON_REGULAR_VISUAL_STUDIO) || defined (__linux__) || defined (__APPLE__) || defined(__FreeBSD__) 
+// Ok. So under Visual Studio, linux, apple and freebsd systems, we have a good chance of having a decent
+// enough strtod. It is not certain, but it is maybe a good enough heuristics. We exclude systems like msys2 
+// or cygwin.
+//
+// Finally, we want to exclude legacy 32-bit systems.
+#ifndef SIMDJSON_IS_32BITS
+// So we only run some of the floating-point tests under 64-bit linux, apple, regular visual studio, freebsd.
+#define TEST_FLOATS
+// Apple and freebsd need a special header, typically.
+#if defined __APPLE__ || defined(__FreeBSD__)
+#  include <xlocale.h> 
+#endif
+
+#endif
+
+#endif
 
 // ulp distance
 // Marc B. Reynolds, 2016-2019
@@ -51,7 +84,7 @@ size_t invalid_count;
 const char *really_bad[] = {"013}", "0x14", "0e]", "0e+]", "0e+-1]"};
 
 bool starts_with(const char *pre, const char *str) {
-  size_t lenpre = strlen(pre);
+  size_t lenpre = std::strlen(pre);
   return strncmp(pre, str, lenpre) == 0;
 }
 
@@ -64,10 +97,20 @@ bool is_in_bad_list(const char *buf) {
   return false;
 }
 
+#ifndef TEST_FLOATS
+// We do not recognize the system, so we do not verify our results.
+void found_invalid_number(const uint8_t *) {}
+#else
 void found_invalid_number(const uint8_t *buf) {
   invalid_count++;
   char *endptr;
-  double expected = strtod((const char *)buf, &endptr);
+#ifdef _WIN32
+  static _locale_t c_locale = _create_locale(LC_ALL, "C");
+  double expected = _strtod_l((const char *)buf, &endptr, c_locale);
+#else
+  static locale_t c_locale = newlocale(LC_ALL_MASK, "C", NULL);
+  double expected = strtod_l((const char *)buf, &endptr, c_locale);
+#endif     
   if (endptr != (const char *)buf) {
     if (!is_in_bad_list((const char *)buf)) {
       printf("Warning: found_invalid_number %.32s whereas strtod parses it to "
@@ -78,22 +121,52 @@ void found_invalid_number(const uint8_t *buf) {
     }
   }
 }
+#endif
 
 void found_integer(int64_t result, const uint8_t *buf) {
   int_count++;
   char *endptr;
   long long expected = strtoll((const char *)buf, &endptr, 10);
   if ((endptr == (const char *)buf) || (expected != result)) {
+#if (!(__MINGW32__) && !(__MINGW64__))
     fprintf(stderr, "Error: parsed %" PRId64 " out of %.32s, ", result, buf);
+#else // mingw is busted since we include #include <inttypes.h> and it will still  not provide PRId64
+    fprintf(stderr, "Error: parsed %lld out of %.32s, ", (long long)result, buf);
+#endif
     fprintf(stderr, " while parsing %s \n", fullpath);
     parse_error |= PARSE_ERROR;
   }
 }
 
+void found_unsigned_integer(uint64_t result, const uint8_t *buf) {
+  int_count++;
+  char *endptr;
+  unsigned long long expected = strtoull((const char *)buf, &endptr, 10);
+  if ((endptr == (const char *)buf) || (expected != result)) {
+#if (!(__MINGW32__) && !(__MINGW64__))
+    fprintf(stderr, "Error: parsed %" PRIu64 " out of %.32s, ", result, buf);
+#else // mingw is busted since we include #include <inttypes.h>
+    fprintf(stderr, "Error: parsed %llu out of %.32s, ", (unsigned long long)result, buf);
+#endif
+    fprintf(stderr, " while parsing %s \n", fullpath);
+    parse_error |= PARSE_ERROR;
+  }
+}
+
+#ifndef TEST_FLOATS
+// We do not recognize the system, so we do not verify our results.
+void found_float(double , const uint8_t *) {}
+#else
 void found_float(double result, const uint8_t *buf) {
   char *endptr;
   float_count++;
-  double expected = strtod((const char *)buf, &endptr);
+#ifdef _WIN32
+  static _locale_t c_locale = _create_locale(LC_ALL, "C");
+  double expected = _strtod_l((const char *)buf, &endptr, c_locale);
+#else
+  static locale_t c_locale = newlocale(LC_ALL_MASK, "C", NULL);
+  double expected = strtod_l((const char *)buf, &endptr, c_locale);
+#endif    
   if (endptr == (const char *)buf) {
     fprintf(stderr,
             "parsed %f from %.32s whereas strtod refuses to parse a float, ",
@@ -101,7 +174,7 @@ void found_float(double result, const uint8_t *buf) {
     fprintf(stderr, " while parsing %s \n", fullpath);
     parse_error |= PARSE_ERROR;
   }
-  if (fpclassify(expected) != fpclassify(result)) {
+  if (std::fpclassify(expected) != std::fpclassify(result)) {
     fprintf(stderr,
             "floats not in the same category expected: %f observed: %f \n",
             expected, result);
@@ -109,20 +182,19 @@ void found_float(double result, const uint8_t *buf) {
     parse_error |= PARSE_ERROR;
     return;
   }
-  // we want to get some reasonable relative accuracy
-  uint64_t ULP = f64_ulp_dist(expected, result);
-  if (f64_ulp_dist(expected, result) > 1) {
+  if (expected != result) {
     fprintf(stderr, "parsed %.128e from \n", result);
     fprintf(stderr, "       %.32s whereas strtod gives\n", buf);
     fprintf(stderr, "       %.128e,", expected);
     fprintf(stderr, " while parsing %s \n", fullpath);
-    fprintf(stderr, " ===========  ULP:  %u,", (unsigned int)ULP);
+    fprintf(stderr, " ===========  ULP:  %u,", (unsigned int)f64_ulp_dist(expected, result));
     parse_error |= PARSE_ERROR;
   }
 }
+#endif
 
-#include "simdjson/jsonparser.h"
-#include "src/stage2_build_tape.cpp"
+#include "simdjson.h"
+#include "simdjson.cpp"
 
 /**
  * Does the file filename ends with the given extension.
@@ -136,7 +208,7 @@ bool validate(const char *dirname) {
   parse_error = 0;
   size_t total_count = 0;
   const char *extension = ".json";
-  size_t dirlen = strlen(dirname);
+  size_t dirlen = std::strlen(dirname);
   struct dirent **entry_list;
   int c = scandir(dirname, &entry_list, 0, alphasort);
   if (c < 0) {
@@ -151,7 +223,7 @@ bool validate(const char *dirname) {
   for (int i = 0; i < c; i++) {
     const char *name = entry_list[i]->d_name;
     if (has_extension(name, extension)) {
-      size_t filelen = strlen(name);
+      size_t filelen = std::strlen(name);
       fullpath = (char *)malloc(dirlen + filelen + 1 + 1);
       strcpy(fullpath, dirname);
       if (needsep) {
@@ -161,24 +233,19 @@ bool validate(const char *dirname) {
         strcpy(fullpath + dirlen, name);
       }
       simdjson::padded_string p;
-      try {
-        simdjson::get_corpus(fullpath).swap(p);
-      } catch (const std::exception &e) {
-        std::cout << "Could not load the file " << fullpath << std::endl;
+      auto error = simdjson::padded_string::load(fullpath).get(p);
+      if (error) {
+        std::cerr << "Could not load the file " << fullpath << std::endl;
         return EXIT_FAILURE;
       }
       // terrible hack but just to get it working
-      simdjson::ParsedJson pj;
-      bool allocok = pj.allocate_capacity(p.size(), 1024);
-      if (!allocok) {
-        std::cerr << "can't allocate memory" << std::endl;
-        return false;
-      }
       float_count = 0;
       int_count = 0;
       invalid_count = 0;
       total_count += float_count + int_count + invalid_count;
-      bool isok = json_parse(p, pj);
+      simdjson::dom::parser parser;
+      auto err = parser.parse(p).error();
+      bool isok = (err == simdjson::error_code::SUCCESS);
       if (int_count + float_count + invalid_count > 0) {
         printf("File %40s %s --- integers: %10zu floats: %10zu invalid: %10zu "
                "total numbers: %10zu \n",
